'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateConfig = updateConfig;

var _utils = require('./utils');

var config = {
  _routeMap: {},

  _isDev: function _isDev() {
    return Boolean(String(process.env.NODE_ENV).match(/dev/i));
  },

  name: 'Lightrail',
  name__type: ['string', 'boolean'],

  host: process.env.HOST || '0.0.0.0',
  host__type: 'string',

  port: process.env.PORT || 8000,
  port__type: 'number',

  trailingSlash: true,
  trailingSlash__type: 'boolean',

  pretty: false,
  pretty__type: 'boolean',

  timeout: 5000,
  timeout__type: 'number',

  payloadLimit: 1048576,
  payloadLimit__type: 'number',

  stripUnknown: true,
  stripUnknown__type: 'boolean',

  protective: true,
  protective__type: 'boolean',

  realIp: true,
  realIp__type: 'boolean',

  documentationUrl: '/docs',
  documentationUrl__type: 'string',

  logs: ['server', 'request', 'error'],
  logs__type: ['array', 'boolean'],

  colors: true,
  colors__type: 'boolean',

  preHandlers: [],
  preHandlers__type: 'array'
};

function updateConfig() {
  var opts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  Object.keys(opts).forEach(function (opt) {
    if (opts[opt] === undefined) return;

    if (opt.match(/^_/)) {
      throw new Error('"' + opt + '" is a read-only value');
    }
    if (!config[opt]) {
      throw new Error('"' + opt + '" isn\'t a valid config key');
    }

    var type = Array.isArray(config[opt + '__type']) ? config[opt + '__type'] : [config[opt + '__type']];

    if (type.indexOf((0, _utils.typeOf)(opts[opt])) === -1) {
      throw new TypeError('"' + opt + '" must be of type "' + type.join(', ') + '"');
    }

    config[opt] = opts[opt];
  });
}

exports.default = config;