'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

exports.sendResponse = sendResponse;

var _stream = require('stream');

var _stream2 = _interopRequireDefault(_stream);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function sendResponse(response, statusCode, data) {
  if (response._redirectTo) {
    var _response$_redirectTo = response._redirectTo;
    var code = _response$_redirectTo.code;
    var url = _response$_redirectTo.url;

    response.statusCode = code;
    response.setHeader('Location', url);
    response.end('Redirecting to ' + url);
    return;
  }

  response.statusCode = parseInt(statusCode, 10) || 200;
  if (_config2.default.name !== false) response.setHeader('Server', _config2.default.name);
  response.setHeader('Cache-Control', 'max-age=0, private, must-revalidate');

  if (_config2.default.protective) {
    response.setHeader('X-Content-Type-Options', 'nosniff');
    response.setHeader('X-Frame-Options', 'deny');
  }

  if (data) {
    if (Buffer.isBuffer(data)) {
      response.setHeader('Content-Type', 'application/octet-stream');
      response.setHeader('Content-Length', data.length);
      response.end(data);
    } else if (data instanceof _stream2.default.Stream) {
      response.setHeader('Content-Type', 'application/octet-stream');
      data.pipe(response);
    } else {
      response.setHeader('Content-Type', 'text/plain');
      if ((typeof data === 'undefined' ? 'undefined' : (0, _typeof3.default)(data)) === 'object') {
        if (data.__noResponse) {
          data = '';
        } else {
          data = _config2.default._isDev() || _config2.default.pretty ? JSON.stringify(data, null, 2) : JSON.stringify(data);
          response.setHeader('Content-Type', 'application/json');
        }
      } else if (data.match(/^<.+>/)) {
        response.setHeader('Content-Type', 'text/html');
      }

      response.setHeader('Content-Length', Buffer.byteLength(data));
      response.end(data);
    }
  } else {
    response.end();
  }
}