'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.expandUrl = expandUrl;
exports.getParamData = getParamData;
exports.getQueryData = getQueryData;
exports.removeUrlQuery = removeUrlQuery;
exports.lookupRoute = lookupRoute;
exports.createRoute = createRoute;
exports.getAllRoutes = getAllRoutes;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function expandUrl(url) {
  var urlType = (0, _utils.typeOf)(url);
  if (urlType === 'regexp') return url;

  if (urlType !== 'string') {
    throw new TypeError('URL must be a string');
  }

  if (url.charAt(0) !== '/') url = '/' + url;
  if (url.slice(-1) === '/') url = url.substring(0, url.length - 1);

  var split = url.split('/');

  return new RegExp('^' + split.map(function (seg, idx) {
    var val = (0, _utils.escapeRegex)(seg);

    if (seg.match(/^:/)) val = '[^\\/\\?]+';

    if (idx === split.length - 1) val += '$';

    return val;
  }).join('\\/'));
}

function getParamData(url, fromUrl) {
  if (typeof url !== 'string') {
    throw new TypeError('URL must be a string');
  }

  var data = {};
  var urlSplit = removeUrlQuery(url).split('/');

  if ((0, _utils.typeOf)(fromUrl) === 'regexp') {
    url.match(fromUrl).slice(1).forEach(function (val, indx) {
      data['$' + (indx + 1)] = val;
    });
  } else {
    fromUrl.split('/').forEach(function (seg, indx) {
      var match = seg.match(/^:(.*)/);
      if (match) data[match[1]] = urlSplit[indx];
    });
  }

  return data;
}

function getQueryData() {
  var request = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  if (typeof request.url !== 'string') {
    throw new TypeError('Param must be an object with a URL key');
  }

  var match = request.url.match(/\?.*$/);
  return match ? _querystring2.default.parse(match[0].substring(1)) : {};
}

function removeUrlQuery() {
  var url = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

  var match = url.match(/\?/) || {};
  if (match.index) url = url.substring(0, match.index);

  return url;
}

function lookupRoute(route) {
  var getAll = arguments.length <= 1 || arguments[1] === undefined ? false : arguments[1];

  var routes = _config2.default._routeMap;

  if (_config2.default.trailingSlash && route.url.slice(-1) === '/' && route.url !== '/') {
    route.url = route.url.substring(0, route.url.length - 1);
  }

  var key = Object.keys(routes).find(function (r) {
    var regex = new RegExp(r);
    return route.url.match(regex);
  });
  if (!key) return;

  var method = route.method.toLowerCase();
  return getAll ? routes[key] : routes[key][method];
}

function createRoute(route) {
  if (Array.isArray(route)) {
    return route.forEach(createRoute);
  }
  if (!route) {
    throw new TypeError('Routes must have a configuration object');
  }
  if ((0, _utils.typeOf)(route.method) !== 'string') {
    throw new TypeError('Route method must be a string');
  }
  if (!(0, _utils.typeOf)(route.url).match(/(string|regexp)/)) {
    throw new TypeError('Route URL must be a string or regex');
  }
  if ((0, _utils.typeOf)(route.handler) !== 'function') {
    throw new TypeError('Route handler must be a function!');
  }

  if (route.method.match(/^options$/i)) {
    throw new Error('Cannot define OPTIONS, this is handled automatically');
  }

  var url = expandUrl(route.url);
  var method = route.method.toLowerCase();

  if (!_config2.default._routeMap[url]) {
    _config2.default._routeMap[url] = {};
  }

  if (_config2.default._routeMap[url][method]) {
    throw new Error('"' + route.method + ' ' + route.url + '" is already defined');
  }

  _config2.default._routeMap[url][method] = route;
}

function getAllRoutes() {
  var data = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  data.dir = _path2.default.resolve(data.cwd || process.cwd(), data.dir || 'routes/**/*.js');

  _glob2.default.sync(data.dir).forEach(function (file) {
    var routes = require(file);
    createRoute(routes.default || routes);
  });
}