'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getIpAddress = getIpAddress;
exports.escapeRegex = escapeRegex;
exports.stringifyRegex = stringifyRegex;
exports.typeOf = typeOf;
exports.promiseWaterfall = promiseWaterfall;

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getIpAddress(request) {
  var remote = request.connection.remoteAddress || request.socket.remoteAddress;
  if (!_config2.default.realIp) return remote;

  var forwarded = request.headers['x-forwarded-for'];
  if (!forwarded) return remote;

  return String(forwarded).split(',')[0].trim();
}

function escapeRegex(str) {
  return str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, '\\$&');
}

function stringifyRegex(regex) {
  var str = String(regex);
  return str.substring(1, str.length - 1);
}

function typeOf(val) {
  var str = Object.prototype.toString.call(val);
  return str.substring(str.indexOf(' ') + 1, str.length - 1).toLowerCase();
}

function promiseWaterfall() {
  var chain = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

  if (!Array.isArray(chain)) {
    throw new TypeError('Promise chain must be an array');
  }

  return chain.reduce(function (prev, next) {
    return prev.then(next);
  }, Promise.resolve());
}