'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _validator = require('validator');

var _validator2 = _interopRequireDefault(_validator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Validation = function () {
  function Validation() {
    (0, _classCallCheck3.default)(this, Validation);

    this.rules = [];

    this.isRequired = false;
  }

  (0, _createClass3.default)(Validation, [{
    key: 'run',
    value: function run(field, val) {
      var where = arguments.length <= 2 || arguments[2] === undefined ? 'body' : arguments[2];

      var errors = [];

      if (!val && !this.isRequired) return [];

      if (!val && this.isRequired) {
        return [{ field: field, message: 'cannot be empty', where: where }];
      }

      this.rules.forEach(function (rule) {
        if (!rule.validate(val)) {
          errors.push({ field: field, message: rule.message, value: val, where: where });
        }
      });

      return errors;
    }
  }, {
    key: 'humanizeLimit',
    value: function humanizeLimit(opts) {
      var space = opts.noSpace ? '' : ' ';

      var leading = opts.min && opts.max ? space + 'between ' : opts.min ? space + 'above ' : opts.max ? space + 'below ' : '';
      var trailing = '' + (opts.min || '') + (opts.min && opts.max ? ' and ' : '') + (opts.max || '');

      return leading + trailing;
    }
  }, {
    key: 'required',
    value: function required() {
      this.isRequired = true;
      return this;
    }
  }, {
    key: 'contains',
    value: function contains(seed) {
      this.rules.push({
        message: 'must contain \'' + seed + '\'',
        validate: function validate(val) {
          return _validator2.default.contains(val, seed);
        }
      });
      return this;
    }
  }, {
    key: 'equals',
    value: function equals(compare) {
      this.rules.push({
        message: 'must be equal to \'' + compare + '\'',
        validate: function validate(val) {
          return _validator2.default.equals(val, compare);
        }
      });
      return this;
    }
  }, {
    key: 'isAfter',
    value: function isAfter(date) {
      date = new Date(date || Date.now());
      this.rules.push({
        message: 'must be after ' + new Date(date).toISOString(),
        validate: function validate(val) {
          return new Date(val).getTime() > date.getTime();
        }
      });
      return this;
    }
  }, {
    key: 'isAlpha',
    value: function isAlpha(locale) {
      this.rules.push({
        message: 'must be contain alpha characters',
        validate: function validate(val) {
          return _validator2.default.isAlpha(val, locale);
        }
      });
      return this;
    }
  }, {
    key: 'isAlphanumeric',
    value: function isAlphanumeric(locale) {
      this.rules.push({
        message: 'must be contain alphanumeric characters',
        validate: function validate(val) {
          return _validator2.default.isAlphanumeric(val, locale);
        }
      });
      return this;
    }
  }, {
    key: 'isAscii',
    value: function isAscii() {
      this.rules.push({
        message: 'must be valid ASCII characters',
        validate: function validate(val) {
          return _validator2.default.isAscii(val);
        }
      });
      return this;
    }
  }, {
    key: 'isBase64',
    value: function isBase64() {
      this.rules.push({
        message: 'must be a valid base64 string',
        validate: function validate(val) {
          return _validator2.default.isBase64(val);
        }
      });
      return this;
    }
  }, {
    key: 'isBefore',
    value: function isBefore(date) {
      date = new Date(date || Date.now());
      this.rules.push({
        message: 'must be before ' + new Date(date).toISOString(),
        validate: function validate(val) {
          return new Date(val).getTime() < date.getTime();
        }
      });
      return this;
    }
  }, {
    key: 'isBoolean',
    value: function isBoolean() {
      this.rules.push({
        message: 'must be a boolean',
        validate: function validate(val) {
          return _validator2.default.isBoolean(String(val));
        }
      });
      return this;
    }
  }, {
    key: 'isCurrency',
    value: function isCurrency(opts) {
      this.rules.push({
        message: 'must be a valid currency',
        validate: function validate(val) {
          return _validator2.default.isCurrency(val, opts);
        }
      });
      return this;
    }
  }, {
    key: 'isDataURI',
    value: function isDataURI() {
      this.rules.push({
        message: 'must be a valid data URI',
        validate: function validate(val) {
          return _validator2.default.isDataURI(val);
        }
      });
      return this;
    }
  }, {
    key: 'isDate',
    value: function isDate() {
      this.rules.push({
        message: 'must be a valid date',
        validate: function validate(val) {
          return _validator2.default.isDate(val);
        }
      });
      return this;
    }
  }, {
    key: 'isDivisibleBy',
    value: function isDivisibleBy(num) {
      this.rules.push({
        message: 'must be divisible by ' + num,
        validate: function validate(val) {
          return _validator2.default.isDivisibleBy(String(val), num);
        }
      });
      return this;
    }
  }, {
    key: 'isEmail',
    value: function isEmail(opts) {
      this.rules.push({
        message: 'must be a valid email address',
        validate: function validate(val) {
          return _validator2.default.isEmail(val, opts);
        }
      });
      return this;
    }
  }, {
    key: 'isFQDN',
    value: function isFQDN(opts) {
      this.rules.push({
        message: 'must be a fully qualified domain name',
        validate: function validate(val) {
          return _validator2.default.isFQDN(val, opts);
        }
      });
      return this;
    }
  }, {
    key: 'isIpAddress',
    value: function isIpAddress(ver) {
      this.rules.push({
        message: 'must be a valid IP address',
        validate: function validate(val) {
          return _validator2.default.isIP(val, ver);
        }
      });
      return this;
    }
  }, {
    key: 'isEnum',
    value: function isEnum(values) {
      this.rules.push({
        message: 'must be one of ' + JSON.stringify(values),
        validate: function validate(val) {
          return _validator2.default.isIn(String(val), values);
        }
      });
      return this;
    }
  }, {
    key: 'isNumber',
    value: function isNumber() {
      var opts = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var msg = this.humanizeLimit(opts);
      this.rules.push({
        message: 'must be an integer' + msg,
        validate: function validate(val) {
          return _validator2.default.isInt(String(val), opts);
        }
      });
      return this;
    }
  }, {
    key: 'isJSON',
    value: function isJSON() {
      this.rules.push({
        message: 'must be valid JSON',
        validate: function validate(val) {
          return _validator2.default.isJSON(val);
        }
      });
      return this;
    }
  }, {
    key: 'isLength',
    value: function isLength(opts) {
      var msg = this.humanizeLimit((0, _extends3.default)({}, opts, { noSpace: true }));
      this.rules.push({
        message: 'must have length ' + msg,
        validate: function validate(val) {
          return _validator2.default.isLength(val, opts);
        }
      });
      return this;
    }
  }, {
    key: 'isLowercase',
    value: function isLowercase() {
      this.rules.push({
        message: 'must be all lowercase letters',
        validate: function validate(val) {
          return _validator2.default.isLowercase(val);
        }
      });
      return this;
    }
  }, {
    key: 'isMACAddress',
    value: function isMACAddress() {
      this.rules.push({
        message: 'must be a valid MAC address',
        validate: function validate(val) {
          return _validator2.default.isMACAddress(val);
        }
      });
      return this;
    }
  }, {
    key: 'isPhoneNumber',
    value: function isPhoneNumber(locale) {
      this.rules.push({
        message: 'must be a valid phone number',
        validate: function validate(val) {
          return _validator2.default.isMobilePhone(val, locale);
        }
      });
      return this;
    }
  }, {
    key: 'isMongoId',
    value: function isMongoId() {
      this.rules.push({
        message: 'must be a valid mongo ID',
        validate: function validate(val) {
          return _validator2.default.isMongoId(val);
        }
      });
      return this;
    }
  }, {
    key: 'isURL',
    value: function isURL(opts) {
      this.rules.push({
        message: 'must be a valid URL',
        validate: function validate(val) {
          return _validator2.default.isURL(val, opts);
        }
      });
      return this;
    }
  }, {
    key: 'isUUID',
    value: function isUUID(ver) {
      this.rules.push({
        message: 'must be a valid UUID',
        validate: function validate(val) {
          return _validator2.default.isUUID(val, ver);
        }
      });
      return this;
    }
  }, {
    key: 'isUppercase',
    value: function isUppercase() {
      this.rules.push({
        message: 'must be all uppercase letters',
        validate: function validate(val) {
          return _validator2.default.isUppercase(val);
        }
      });
      return this;
    }
  }, {
    key: 'isWhitelisted',
    value: function isWhitelisted(chars) {
      this.rules.push({
        message: 'must be whitelisted',
        validate: function validate(val) {
          return _validator2.default.isWhitelisted(String(val), chars);
        }
      });
      return this;
    }
  }, {
    key: 'matches',
    value: function matches(pattern, mods) {
      this.rules.push({
        message: 'must be valid',
        validate: function validate(val) {
          return _validator2.default.matches(val, pattern, mods);
        }
      });
      return this;
    }
  }]);
  return Validation;
}();

exports.default = Validation;